@@ -7,16 +7,30 @@ if sys.version_info < (2, 5):
             "'python2.6 cndcc <arguments>'.")
 
 import ply
-from pycparser.c_lexer import CLexer as CLexerBase
-from pycparser.gnu_c_parser import GnuCParser as CParserBase
+from pycparserext.ext_c_lexer import (
+        add_lexer_keywords,
+        GnuCLexer as GnuCLexerBase,
+        OpenCLCLexer as OpenCLCLexerBase,
+        )
 from pycparser import c_ast
-from pycparser.c_generator import CGenerator as CGeneratorBase
+from pycparserext.ext_c_parser import (
+        GnuCParser as GnuCParserBase,
+        OpenCLCParser as OpenCLCParserBase,
+        )
+
+from pycparserext.ext_c_generator import (
+        GnuCGenerator as GnuCGeneratorBase,
+        OpenCLCGenerator as OpenCLCGeneratorBase,
+        )
+
 import sys
 
 
 
 
 
+import cnd.version
+
 CND_HELPERS = """
 #define CND_ALLOC_HEAP(type, name) \
   name = malloc(nitemsof(name)*sizeof(type));
@@ -30,23 +44,33 @@ CND_HELPERS = """
 
 #define CND_FOR_AXIS(it_var, name, ax_index) \
   for (long it_var = lboundof(name, ax_index); it_var < puboundof(name, ax_index); ++it_var)
-"""
+
+#define CND_VERSION_MAJOR %(major_ver)d
+#define CND_VERSION_MINOR %(minor_ver)d
+#define CND_VERSION_TEXT %(text_ver)s
+""" % dict(
+        major_ver=cnd.version.VERSION[0],
+        minor_ver=cnd.version.VERSION[1],
+        text_ver=cnd.version.VERSION_TEXT,
+        )
 
 
 
 
+# {{{ lexers
+
+class GnuCndLexer(GnuCLexerBase):
+    pass
+
+class OpenCLCndLexer(OpenCLCLexerBase):
+    pass
+
 cnd_keywords = ("dimension",)
-class CNdArrayLexer(CLexerBase):
 
-    keywords = CLexerBase.keywords + tuple(
-            kw.upper() for kw in cnd_keywords)
+add_lexer_keywords(GnuCndLexer, cnd_keywords)
+add_lexer_keywords(OpenCLCndLexer, cnd_keywords)
 
-    keyword_map = CLexerBase.keyword_map.copy()
-    keyword_map.update(dict(
-        (kw, kw.upper()) for kw in cnd_keywords))
-
-    tokens = CLexerBase.tokens + tuple(
-            kw.upper() for kw in cnd_keywords)
+# }}}
 
 # {{{ parenthesis insertion
 
@@ -114,6 +138,8 @@ def insert_parens_in_brackets(filename, s):
 
 
 
+# {{{ AST helper objects
+
 class SingleDim(object):
     def __init__(self, layout, start, end, stride, leading_dim):
         self.end = end
@@ -138,7 +164,7 @@ class SingleDim(object):
         self.leading_dim = leading_dim
 
     def __repr__(self):
-        generator = CGenerator()
+        generator = GnuCGenerator()
 
         def stringify(x):
             if x is None:
@@ -166,52 +192,11 @@ class DimensionDecl(c_ast.Node):
 
     attr_names = ("name", "layout", "dims")
 
-class CNdArrayParser(CParserBase):
-    def __init__(self, yacc_debug=False):
+# }}}
 
-        self.clex = CNdArrayLexer(
-            error_func=self._lex_error_func,
-            type_lookup_func=self._lex_type_lookup_func)
+# {{{ parsers
 
-        self.clex.build()
-        self.tokens = self.clex.tokens
-
-        rules_with_opt = [
-            'abstract_declarator',
-            'assignment_expression',
-            'declaration_list',
-            'declaration_specifiers',
-            'designation',
-            'expression',
-            'identifier_list',
-            'init_declarator_list',
-            'parameter_type_list',
-            'specifier_qualifier_list',
-            'block_item_list',
-            'type_qualifier_list',
-            'struct_declarator_list'
-        ]
-
-        for rule in rules_with_opt:
-            self._create_opt_rule(rule)
-
-        self.cparser = ply.yacc.yacc(
-            module=self,
-            start='translation_unit',
-            debug=yacc_debug, write_tables=False)
-
-    def parse(self, text, filename='', debuglevel=0,
-            initial_type_symbols=set()):
-        self.clex.filename = filename
-        self.clex.reset_lineno()
-
-        # _scope_stack[-1] is the current (topmost) scope.
-
-        self._scope_stack = [set(initial_type_symbols)]
-        if not text or text.isspace():
-            return c_ast.FileAST([])
-        else:
-            return self.cparser.parse(text, lexer=self.clex, debug=debuglevel)
+class CndParserBase(object):
 
     # {{{ hack around [()]
 
@@ -259,9 +244,9 @@ class CNdArrayParser(CParserBase):
         if layout is not None:
             layout = layout[1:-1]
 
-        if layout is None or layout == "c":
+        if layout is None:
             layout = "c"
-        elif layout == "fortran":
+        elif layout in ["c" , "col-major", "row-major", "fortran"]:
             pass
         else:
             raise RuntimeError("invalid  array layout '%s'" % layout)
@@ -325,16 +310,26 @@ class CNdArrayParser(CParserBase):
         p[0] = (p[1], p[3], p[5], p[7])
 
 
+class GnuCndParser(CndParserBase, GnuCParserBase):
+    lexer_class = GnuCndLexer
+
+class OpenCLCndParser(CndParserBase, OpenCLCParserBase):
+    lexer_class = OpenCLCndLexer
+
+# }}}
+
 
 
 
 class SyntaxError(RuntimeError):
     pass
 
-class CGenerator(CGeneratorBase):
+# {{{ generators
+
+class CndGeneratorMixin(object):
     def __init__(self):
-        CGeneratorBase.__init__(self)
         self.dim_decl_stack = [{}]
+        self.generate_line_directives = True
 
     def visit_DimensionDecl(self, n):
         decl_stack = self.dim_decl_stack[-1]
@@ -344,15 +339,21 @@ class CGenerator(CGeneratorBase):
         decl_stack[n.name] = n
         return ""
 
+    # overrides base to treat dim_decl_stack
     def visit_Compound(self, n):
         s = self._make_indent() + '{\n'
         self.indent_level += 2
 
         dim_decls = self.dim_decl_stack[-1].copy()
         self.dim_decl_stack.append(dim_decls)
-        for stmt in n.block_items:
-            s += "# %d \"%s\"\n%s" % (
-                stmt.coord.line, stmt.coord.file, ''.join(self._generate_stmt(stmt)))
+
+        if n.block_items: # may be None
+            for stmt in n.block_items:
+                if self.generate_line_directives:
+                    s += "# %d \"%s\"\n" % (
+                        stmt.coord.line, stmt.coord.file)
+
+                s += ''.join(self._generate_stmt(stmt))
 
         self.dim_decl_stack.pop()
 
@@ -368,12 +369,11 @@ class CGenerator(CGeneratorBase):
 
         axis_numbers = range(len(indices))
 
-        if dim_decl.layout == "c":
-            dim_it = zip(indices, dim_decl.dims, axis_numbers)
-        elif dim_decl.layout == "fortran":
+        is_col_major = dim_decl.layout in ["fortran", "col-major"]
+        if is_col_major:
             dim_it = zip(indices[::-1], dim_decl.dims[::-1], axis_numbers[::-1])
         else:
-            raise SyntaxError("invalid  array layout '%s'" % dim_decl.layout)
+            dim_it = zip(indices, dim_decl.dims, axis_numbers)
 
         access = None
         for idx, dim, axis in dim_it:
@@ -407,12 +407,12 @@ class CGenerator(CGeneratorBase):
             if isinstance(n.subscript, c_ast.ExprList):
                 indices = n.subscript.exprs
             else:
-                indices = n.subscript
+                indices = (n.subscript,)
 
             if dim_decl is not None:
                 return self.generate_array_ref(dim_decl, n.name.name, indices, n.coord)
 
-        return CGeneratorBase.visit_ArrayRef(self, c_ast.ArrayRef(
+        return self.generator_base_class.visit_ArrayRef(self, c_ast.ArrayRef(
             n.name, n.subscript, n.coord))
 
     def visit_FuncCall(self, n):
@@ -486,7 +486,7 @@ class CGenerator(CGeneratorBase):
                 elif name == "uboundof":
                     result = dim_decl.dims[axis].end
                 elif name == "puboundof":
-                    if dim_decl.layout == "c":
+                    if dim_decl.layout in ["c", "row-major", "col-major"]:
                         result = dim_decl.dims[axis].end
                     elif dim_decl.layout == "fortran":
                         result = c_ast.BinaryOp("+",
@@ -503,15 +503,30 @@ class CGenerator(CGeneratorBase):
 
                 if result is None:
                     raise SyntaxError("no value available for '%s' at %s" % (
-                        CGeneratorBase.visit_FuncCall(self, n), n.coord))
+                        self.generator_base_class.visit_FuncCall(self, n), n.coord))
 
                 return self.visit(result)
 
-        return CGeneratorBase.visit_FuncCall(self, n)
+        return self.generator_base_class.visit_FuncCall(self, n)
 
 
 
 
+class GnuCGenerator(CndGeneratorMixin, GnuCGeneratorBase):
+    generator_base_class = GnuCGeneratorBase
+    def __init__(self):
+        GnuCGeneratorBase.__init__(self)
+        CndGeneratorMixin.__init__(self)
+
+class OpenCLCGenerator(CndGeneratorMixin, OpenCLCGeneratorBase):
+    generator_base_class = OpenCLCGeneratorBase
+    def __init__(self):
+        OpenCLCGeneratorBase.__init__(self)
+        CndGeneratorMixin.__init__(self)
+
+# }}}
+
+# {{{ run helpers
 
 class ExecError(RuntimeError):
     pass
@@ -614,7 +629,6 @@ def preprocess_source(source, cpp, options):
 
 
 
-INITIAL_TYPE_SYMBOLS = ["__builtin_va_list"]
 PREAMBLE = [
         CND_HELPERS,
         "#define __extension__ /*empty*/", # FIXME
@@ -672,14 +686,13 @@ def run_standalone():
 
     #print "preprocessed source in ", write_temp_file(src, ".c")
 
-    parser = CNdArrayParser()
-    ast = parser.parse(src, filename=in_file,
-            initial_type_symbols=INITIAL_TYPE_SYMBOLS)
+    parser = GnuCndParser()
+    ast = parser.parse(src, filename=in_file)
     if options.ast:
         ast.show()
         return
 
-    generator = CGenerator()
+    generator = GnuCGenerator()
 
     if options.output is not None:
         outf = open(options.output, "wt")
@@ -728,11 +741,10 @@ def run_as_compiler_frontend():
 
                 #print "preprocessed source in ", write_temp_file(src, ".c")
 
-                parser = CNdArrayParser()
-                ast = parser.parse(src, filename=arg,
-                        initial_type_symbols=INITIAL_TYPE_SYMBOLS)
+                parser = GnuCndParser()
+                ast = parser.parse(src, filename=arg)
 
-                generator = CGenerator()
+                generator = GnuCGenerator()
 
                 gen_src_file = write_temp_file(generator.visit(ast), ".c")
                 new_argv.append(gen_src_file)
@@ -766,3 +778,19 @@ def run_as_compiler_frontend():
     finally:
         for tempf in temp_files:
             os.unlink(tempf)
+
+
+
+
+def transform_cl(src, filename=None):
+    parser = OpenCLCndParser()
+    ast = parser.parse(src, filename=filename)
+
+    generator = OpenCLCGenerator()
+    generator.generate_line_directives = False
+    return generator.visit(ast)
+
+
+# }}}
+
+# vim: fdm=marker
